package TVListings::now;

use strict;
use warnings;

use Date::Parse;
use Apache::Constants qw/:http/;
use POSIX;

use TVListings::Config;
use TVListings::DB qw/:default SQL_NOW/;
use TVListings::grid qw/:render/;

sub render {
  my ($class, $r, $q, $dbh) = @_;
  
  my $sth = $dbh->prepare_cached(SQL_NOW);
  $sth->execute;
  my $data= $sth->fetchall_arrayref({});
  
  my @now = localtime(time);
	my $nowstr = strftime("%H:%M", @now);

  my $out = pagehead_mini($r, $q, "On TV Now - $nowstr");
  
  my $now_patts = get_now_patts($dbh);
  
  $out .= '<table class="tvlistings nowlistings">';
  
  my $prg_interest = get_all_prg_interest($dbh);
  my $cicons = get_all_cicon($dbh);
  
  $out .= qq{<tr><th colspan="2">Channel</th><th>Show</th></tr>\n};
  
  for my $row (@$data) {
    # skip junky shit to make sidebar short
    next if $prg_interest->{$row->{prg_title}}
      and $prg_interest->{$row->{prg_title}} eq 'bad';
    my %rtimes = map {$_ => str2time($row->{$_})} qw/prg_start prg_stop/;
    $rtimes{blk_start} = $rtimes{prg_start};
    $rtimes{blk_end} = $rtimes{prg_stop};
    $out .= render_row_header($r, $row, $cicons, 2);
    $out .= render_grid_cell($r, $row, $now_patts->{$row->{prg_oid}},
      $prg_interest->{$row->{prg_title}}, '', \%rtimes);
  }
  $out .= '</tr></table>';
  my $rtime = 300 - (time % 300);
  $out .= pagefoot_mini($r, $q);
  
  return {
    hdrs => [['Refresh', $rtime]],
    status => HTTP_OK,
    text => $out,
  };
}


1;
