
SET client_encoding = 'SQL_ASCII';
SET check_function_bodies = false;
CREATE SCHEMA @@SCHEMA@@;
SET search_path = @@SCHEMA@@, pg_catalog;
CREATE FUNCTION longer(character varying, character varying) RETURNS character varying
    AS 'select case when length($2) > length($1) then $2 else $1 end;'
    LANGUAGE sql IMMUTABLE;

CREATE AGGREGATE longest (
    BASETYPE = character varying,
    SFUNC = longer,
    STYPE = character varying,
    INITCOND = ''
);

CREATE FUNCTION round_5min(timestamp with time zone) RETURNS timestamp with time zone
    AS 'DECLARE
	result timestamptz;
	fixmin int4;
BEGIN
	result := $1;
	result := date_trunc(''minute'', result);
	fixmin := extract(minute from result);
	fixmin := fixmin % 5;
	result := result - (fixmin || '' minutes'')::interval;
	return result;
END;'
    LANGUAGE plpgsql STABLE;

CREATE FUNCTION now_5min() RETURNS timestamp with time zone
    AS 'select @@SCHEMA@@.round_5min(now());'
    LANGUAGE sql STABLE;

CREATE FUNCTION grid_end() RETURNS timestamp with time zone
    AS 'select @@SCHEMA@@.grid_endx(@@SCHEMA@@.now_5min())'
    LANGUAGE sql STABLE;

CREATE FUNCTION roundup_5min(timestamp with time zone) RETURNS timestamp with time zone
    AS 'DECLARE
	result timestamptz;
	fixmin int4;
BEGIN
	result := $1;
	result := date_trunc(''minute'', result);
	fixmin := extract(minute from result);
	fixmin := fixmin % 5;
	if fixmin > 0 then
		result := result + ((5 - fixmin) || '' minutes'')::interval;
	end if;
	return result;
END;'
    LANGUAGE plpgsql STABLE;

CREATE FUNCTION blk_start(timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 < @@SCHEMA@@.now_5min()
then @@SCHEMA@@.now_5min()
else @@SCHEMA@@.round_5min($1) end'
    LANGUAGE sql STABLE;

CREATE FUNCTION blk_end(timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 > @@SCHEMA@@.grid_end() then @@SCHEMA@@.grid_end() else @@SCHEMA@@.roundup_5min($1) end'
    LANGUAGE sql STABLE;

CREATE TABLE channels (
    chan_oid bigserial NOT NULL,
    chan_id character varying(255) NOT NULL
) WITHOUT OIDS;

CREATE TABLE chan_attrs (
    catt_oid bigserial NOT NULL,
    chan_oid bigint NOT NULL,
    catt_name character varying(50) NOT NULL,
    catt_value character varying(255) NOT NULL
) WITHOUT OIDS;

CREATE TABLE icons (
    icn_oid bigserial NOT NULL,
    icn_src character varying(255) NOT NULL,
    icn_width integer,
    icn_height integer
) WITHOUT OIDS;

CREATE TABLE chan_icons (
    icn_oid bigint,
    icn_src character varying(255),
    icn_width integer,
    icn_height integer,
    chan_oid bigint NOT NULL
)
INHERITS (icons) WITHOUT OIDS;

CREATE VIEW chan_long_name AS
    SELECT chan_attrs.chan_oid, longest(chan_attrs.catt_value) AS longest_display_name FROM chan_attrs WHERE ((chan_attrs.catt_name)::text = 'display-name'::text) GROUP BY chan_attrs.chan_oid;

CREATE TABLE programmes (
    prg_oid bigserial NOT NULL,
    prg_start timestamp with time zone NOT NULL,
    prg_stop timestamp with time zone,
    chan_oid bigint NOT NULL
) WITHOUT OIDS;

CREATE TABLE prg_icons (
    icn_oid bigint,
    icn_src character varying(255),
    icn_width integer,
    icn_height integer,
    prg_oid bigint NOT NULL
)
INHERITS (icons) WITHOUT OIDS;

CREATE TABLE prg_attrs (
    patt_oid bigserial NOT NULL,
    prg_oid bigint NOT NULL,
    patt_name character varying(50) NOT NULL,
    patt_subname character varying(50),
    patt_lang character varying(2),
    patt_value character varying(255) NOT NULL
) WITHOUT OIDS;

CREATE TABLE patt_icons (
    icn_oid bigint,
    icn_src character varying(255),
    icn_width integer,
    icn_height integer,
    patt_oid bigint NOT NULL
)
INHERITS (icons) WITHOUT OIDS;

CREATE VIEW chan_best_name AS
    SELECT chan_attrs.chan_oid, ("substring"((chan_attrs.catt_value)::text, '^\\d+'::text))::integer AS chan_number, "substring"((chan_attrs.catt_value)::text, '^\\d+ (.*)$'::text) AS chan_name FROM chan_attrs WHERE (((chan_attrs.catt_name)::text = 'display-name'::text) AND ((chan_attrs.catt_value)::text ~ similar_escape('[0-9]+ [^ ]+$'::text, NULL::text)));

CREATE VIEW prg_long_title AS
    SELECT prg_attrs.prg_oid, prg_attrs.patt_value AS longest_prg_title FROM prg_attrs WHERE ((prg_attrs.patt_name)::text = 'title'::text);

CREATE VIEW shows AS
    SELECT prg_oid, prg_start, prg_stop, prg_long_title.longest_prg_title AS prg_title, chan_oid, chan_number, chan_name FROM (((programmes NATURAL JOIN channels) NATURAL JOIN chan_best_name) NATURAL JOIN prg_long_title);

CREATE VIEW active_shows AS
    SELECT shows.prg_oid, shows.prg_start, shows.prg_stop, shows.prg_title, shows.chan_oid, shows.chan_number, shows.chan_name FROM shows WHERE ((((shows.prg_start >= (now() - '08:00:00'::interval)) AND (shows.prg_start < grid_end())) AND (shows.prg_stop > now_5min())) AND (shows.prg_stop <= (grid_end() + '08:00:00'::interval)));

CREATE VIEW active_shows_grid AS
    SELECT active_shows.prg_oid, active_shows.prg_start, active_shows.prg_stop, blk_start(active_shows.prg_start) AS blk_start, blk_end(active_shows.prg_stop) AS blk_end, active_shows.prg_title, active_shows.chan_oid, active_shows.chan_number, active_shows.chan_name FROM active_shows ORDER BY active_shows.chan_number, active_shows.chan_name, active_shows.prg_start;

CREATE TABLE prg_interest (
    prg_title character varying(255) NOT NULL,
    pint_class character varying(50) NOT NULL
) WITHOUT OIDS;

CREATE TABLE cv_interest (
    pint_class character varying(255) NOT NULL,
    pint_order integer DEFAULT 2147483647 NOT NULL,
    weight integer,
    CONSTRAINT cv_pint_class_chk CHECK (((pint_class)::text ~ '^[a-z]+$'::text))
) WITHOUT OIDS;

CREATE VIEW active_prgs AS
    SELECT programmes.prg_oid, programmes.prg_start, programmes.prg_stop, programmes.chan_oid FROM programmes WHERE ((((programmes.prg_start >= (now() - '08:00:00'::interval)) AND (programmes.prg_start < grid_end())) AND (programmes.prg_stop > now_5min())) AND (programmes.prg_stop <= (grid_end() + '08:00:00'::interval)));

CREATE VIEW shows_grid AS
    SELECT shows.prg_oid, shows.prg_start, shows.prg_stop, blk_start(shows.prg_start) AS blk_start, blk_end(shows.prg_stop) AS blk_end, shows.prg_title, shows.chan_oid, shows.chan_number, shows.chan_name FROM shows ORDER BY shows.chan_number, shows.chan_name, shows.prg_start;

CREATE VIEW chan_quality AS
    SELECT cq.chan_name, cq.pint_class, cq.totaltime, (cq.totaltime * (weight)::double precision) AS weightedtime FROM ((SELECT chan_name, COALESCE(pint_class, 'normal'::character varying) AS pint_class, sum((prg_stop - prg_start)) AS totaltime FROM (shows NATURAL LEFT JOIN prg_interest) GROUP BY chan_name, COALESCE(pint_class, 'normal'::character varying)) cq NATURAL JOIN cv_interest) ORDER BY cq.chan_name, cq.pint_class;

CREATE VIEW shows_now AS
    SELECT shows.prg_oid, shows.prg_start, shows.prg_stop, shows.prg_title, shows.chan_oid, shows.chan_number, shows.chan_name FROM shows WHERE (((shows.prg_start >= (now() - '08:00:00'::interval)) AND (shows.prg_start <= now())) AND (shows.prg_stop > now())) ORDER BY shows.chan_number, shows.chan_name, shows.prg_start;

CREATE VIEW prgs_now AS
    SELECT programmes.prg_oid, programmes.prg_start, programmes.prg_stop, programmes.chan_oid FROM programmes WHERE (((programmes.prg_start >= (now() - '08:00:00'::interval)) AND (programmes.prg_start <= now())) AND (programmes.prg_stop > now()));

CREATE FUNCTION grid_endx(timestamp with time zone) RETURNS timestamp with time zone
    AS 'DECLARE
	res timestamptz;
	tmins int4;
BEGIN
	res := @@SCHEMA@@.roundup_5min($1);
	return res + ''2 hours''::interval;
END;'
    LANGUAGE plpgsql STABLE;

CREATE FUNCTION shows_grid_then(timestamp with time zone) RETURNS SETOF shows_grid
    AS 'SELECT s.prg_oid, s.prg_start, s.prg_stop,
	@@SCHEMA@@.blk_startx(s.prg_start, $1) AS blk_start,
	@@SCHEMA@@.blk_endx(s.prg_stop, $1) AS blk_end,
	s.prg_title, s.chan_oid, s.chan_number, s.chan_name
FROM @@SCHEMA@@.shows s
WHERE s.prg_start >= ($1 - ''08:00:00''::interval)
	AND s.prg_start < @@SCHEMA@@.grid_endx($1)
	AND s.prg_stop > @@SCHEMA@@.round_5min($1)
	AND s.prg_stop <= (@@SCHEMA@@.grid_endx($1) + ''08:00:00''::interval)
ORDER BY s.chan_number, s.chan_name, s.prg_start;'
    LANGUAGE sql STABLE;

CREATE FUNCTION blk_endx(timestamp with time zone, timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 > @@SCHEMA@@.grid_endx($2)
then @@SCHEMA@@.grid_endx($2)
else @@SCHEMA@@.roundup_5min($1) end'
    LANGUAGE sql STABLE;

CREATE FUNCTION blk_startx(timestamp with time zone, timestamp with time zone) RETURNS timestamp with time zone
    AS 'select case when $1 < @@SCHEMA@@.round_5min($2)
then @@SCHEMA@@.round_5min($2)
else @@SCHEMA@@.round_5min($1) end'
    LANGUAGE sql STABLE;

CREATE FUNCTION prgs_then(timestamp with time zone) RETURNS SETOF programmes
    AS 'SELECT *
   FROM @@SCHEMA@@.programmes p
WHERE p.prg_start >= ($1 - ''08:00:00''::interval)
AND p.prg_start < @@SCHEMA@@.grid_endx($1)
AND p.prg_stop > @@SCHEMA@@.round_5min($1)
AND p.prg_stop <= (@@SCHEMA@@.grid_endx($1) + ''08:00:00''::interval)'
    LANGUAGE sql STABLE;

CREATE FUNCTION prg_attrs_then(timestamp with time zone) RETURNS SETOF prg_attrs
    AS 'SELECT pa.* 
   FROM @@SCHEMA@@.prg_attrs pa
   natural join @@SCHEMA@@.programmes p
WHERE p.prg_start >= ($1 - ''08:00:00''::interval)
AND p.prg_start < @@SCHEMA@@.grid_endx($1)
AND p.prg_stop > @@SCHEMA@@.round_5min($1)
AND p.prg_stop <= (@@SCHEMA@@.grid_endx($1) + ''08:00:00''::interval)'
    LANGUAGE sql STABLE;

CREATE INDEX ix_prg_start_stop ON programmes USING btree (prg_start, prg_stop);

ALTER TABLE programmes CLUSTER ON ix_prg_start_stop;

CREATE INDEX ix_cicons_chan_oid ON chan_icons USING btree (chan_oid);

CREATE INDEX ix_patt_search ON prg_attrs USING btree (prg_oid, patt_name, patt_value);

ALTER TABLE prg_attrs CLUSTER ON ix_patt_search;

CREATE INDEX ix_pint_class ON prg_interest USING btree (pint_class);

CREATE INDEX ix_catt_best_name ON chan_attrs USING btree (chan_oid, catt_value) WHERE (((catt_name)::text = 'display-name'::text) AND ((catt_value)::text ~ '^[0-9]+ [A-Z]+$'::text));

CREATE INDEX ix_catt_search ON chan_attrs USING btree (chan_oid, catt_name, catt_value);

ALTER TABLE chan_attrs CLUSTER ON ix_catt_search;

ALTER TABLE ONLY channels
    ADD CONSTRAINT channels_pkey PRIMARY KEY (chan_oid);

ALTER TABLE channels CLUSTER ON channels_pkey;

ALTER TABLE ONLY channels
    ADD CONSTRAINT channels_chan_id_key UNIQUE (chan_id);

ALTER TABLE ONLY chan_attrs
    ADD CONSTRAINT channel_attrs_pkey PRIMARY KEY (catt_oid);

ALTER TABLE ONLY icons
    ADD CONSTRAINT icons_pkey PRIMARY KEY (icn_oid);

ALTER TABLE ONLY programmes
    ADD CONSTRAINT programmes_pkey PRIMARY KEY (prg_oid);

ALTER TABLE ONLY prg_attrs
    ADD CONSTRAINT prg_attrs_pkey PRIMARY KEY (patt_oid);

ALTER TABLE ONLY prg_interest
    ADD CONSTRAINT prg_interest_pkey PRIMARY KEY (prg_title);

ALTER TABLE prg_interest CLUSTER ON prg_interest_pkey;

ALTER TABLE ONLY cv_interest
    ADD CONSTRAINT cv_interest_pkey PRIMARY KEY (pint_class);

ALTER TABLE ONLY programmes
    ADD CONSTRAINT prg_chan_ref FOREIGN KEY (chan_oid) REFERENCES channels(chan_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY prg_icons
    ADD CONSTRAINT prg_icon_prg_ref FOREIGN KEY (prg_oid) REFERENCES programmes(prg_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY prg_attrs
    ADD CONSTRAINT patt_prg_ref FOREIGN KEY (prg_oid) REFERENCES programmes(prg_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY patt_icons
    ADD CONSTRAINT patt_icon_patt_ref FOREIGN KEY (patt_oid) REFERENCES prg_attrs(patt_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY chan_icons
    ADD CONSTRAINT chan_icon_chan_ref FOREIGN KEY (chan_oid) REFERENCES channels(chan_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY chan_attrs
    ADD CONSTRAINT catt_chan_ref FOREIGN KEY (chan_oid) REFERENCES channels(chan_oid) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE ONLY prg_interest
    ADD CONSTRAINT pint_cv_ref FOREIGN KEY (pint_class) REFERENCES cv_interest(pint_class) ON UPDATE RESTRICT ON DELETE RESTRICT;

COMMENT ON FUNCTION longer(character varying, character varying) IS 'Returns the longer of two input strings';

COMMENT ON AGGREGATE longest(character varying) IS 'Chooses the string-wise longset value from a set';

COMMENT ON FUNCTION round_5min(timestamp with time zone) IS 'Rounds a time to the previous five minute mark';

COMMENT ON FUNCTION now_5min() IS 'Rounds current time to previous 5 minute mark';

COMMENT ON FUNCTION grid_end() IS 'Returns the time at which the current grid should end';

COMMENT ON FUNCTION roundup_5min(timestamp with time zone) IS 'Rounds a time to the next five minute mark';

COMMENT ON FUNCTION blk_start(timestamp with time zone) IS 'Returns the ''programming block'' start for a given time - the time rounded to a five minute mark, shifted forwards to be at least now';

COMMENT ON FUNCTION blk_end(timestamp with time zone) IS 'Returns the ''programming block'' end time - the time rounted to a five minute mark, truncated down to be not more than the grid end for now.';

COMMENT ON VIEW chan_long_name IS 'Longest name for a channel (unused)';

COMMENT ON VIEW chan_best_name IS 'Best name for a channel to present to user';

COMMENT ON VIEW prg_long_title IS 'Longest title for a show';

COMMENT ON VIEW shows IS 'Shows - program with title and channel info';

COMMENT ON VIEW active_shows IS 'Shows in the active time range.
Extra conditions to optimize index usage';

COMMENT ON VIEW active_shows_grid IS 'Active shows with grid information added';

COMMENT ON VIEW active_prgs IS 'Programs in the active time range.
Extra conditions to optimize index usage';

COMMENT ON VIEW shows_grid IS 'Shows with grid information added';

COMMENT ON VIEW chan_quality IS 'Attempts to estimate the overall quality of a channel by tallying the ratings of the shows it airs';

COMMENT ON VIEW shows_now IS 'Shows that are on right now';

COMMENT ON VIEW prgs_now IS 'Programmes that are showing right now';

COMMENT ON FUNCTION grid_endx(timestamp with time zone) IS 'Returns the time at which a grid should end when it starts at the passed time.';

COMMENT ON FUNCTION shows_grid_then(timestamp with time zone) IS 'Fetches a table like active_shows_grid, but for a grid starting other than now';

COMMENT ON FUNCTION blk_endx(timestamp with time zone, timestamp with time zone) IS 'The programming block end for a time and a grid start';

COMMENT ON FUNCTION blk_startx(timestamp with time zone, timestamp with time zone) IS 'The programming block start for a given time and grid start';

COMMENT ON FUNCTION prgs_then(timestamp with time zone) IS 'Fetches programmes active in a grid starting at the given time';

COMMENT ON FUNCTION prg_attrs_then(timestamp with time zone) IS 'Fetches programme attrs for prgs on during a grid block starting at the given time';

SET client_encoding = 'SQL_ASCII';
SET check_function_bodies = false;

SET search_path = @@SCHEMA@@, pg_catalog;
COPY cv_interest (pint_class, pint_order, weight) FROM stdin;
good	1	2
normal	2	0
crap	3	0
bad	4	-1
\.
